let Cc = Components.classes, Ci = Components.interfaces;

let ExtensionPreferencesMenu = {
	startup: function() {
		window.removeEventListener('load', function EPMload() { ExtensionPreferencesMenu.startup() });

		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.addAddonListener(this);

		Components.utils.import("resource://gre/modules/Services.jsm");
		this._branch = Services.prefs.getBranch("extensions.ExtensionPreferencesMenu.");
		this._branch.addObserver("", this, false);

		"CustomizableUI" in window ? CustomizableUI.addListener(this) : window.addEventListener('customizationchange', function EPMsetup() { ExtensionPreferencesMenu.setup() });

		if (this._branch.getBoolPref("FirstRun")) {
			this.onFirstRun();
		}

		this.setup();
	},

	onWidgetAdded: function(aWidgetId) {
		if (aWidgetId == 'extensionPrefsMenu-button')
			this.setup();
	},

	onWidgetRemoved: function(aWidgetId) {
		if (aWidgetId == 'extensionPrefsMenu-button')
			this.setup();
	},

	onAreaNodeRegistered: function() this.setup(),

	onAreaNodeUnregistered: function() this.setup(),

	onDisabled: function() this.setup(),

	onEnabled: function() this.setup(),

	onUninstalled: function() this.setup(),

	onInstalled: function() this.setup(),

	observe: function(aSubject, aTopic, aData) {
		if (aTopic == "nsPref:changed")
			this.setup();
	},

	onFirstRun: function() {
		this._branch.setBoolPref("FirstRun", false);
	},

	setup: function() {
		let EPToolsmenu = document.getElementById("extensionPrefsMenu");
		EPToolsmenu.hidden = !Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.InToolsMenu");
		if (Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowIcons")) {
			EPToolsmenu.setAttribute("class", "menu-iconic");
			EPToolsmenu.setAttribute("style", "list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric-16.png)");
		}
		else
			EPToolsmenu.setAttribute("class", "menu");

		this.populate(EPToolsmenu.firstChild);

		let EPAppmenu = document.getElementById("appmenu_extensionPrefs");
		if (EPAppmenu) {
			EPAppmenu.collapsed = !Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.InAppMenu");
			let EPMorigAddonMenu = document.getElementById("appmenu_addons");
			if (EPMorigAddonMenu)
				EPMorigAddonMenu.collapsed = !EPAppmenu.collapsed;

			this.populate(EPAppmenu.firstChild);
		}

		let EPCTRmenu = document.getElementById("EPM_CTR");
		if (EPCTRmenu) {
			EPCTRmenu.collapsed = !Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.InAppMenu");
			let EPMorigAddonMenu = document.getElementById("appmenu_addons");
			if (EPMorigAddonMenu)
				EPMorigAddonMenu.collapsed = !EPCTRmenu.collapsed;

			this.populate(document.getElementById("EPM_CTRpopup"));
		}

		let EPMToolbarButton = document.getElementById("extensionPrefsMenu-button");
		if (EPMToolbarButton)
			this.populate(EPMToolbarButton.firstChild);
	},

	populate: function(aParent) {
		let item, MenuIconURL, EPMShowIt = true, EPMBlackListArray = [];
		let EPshowVersion = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowVersion");
		let EPshowIcon = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowIcons");
		let EPMshowAll = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowAll");
		let EPMshowDisabled = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowDisabled");
		let EPMshowIncompatible = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowIncompatible");
		let EPMquickActions = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.QuickActions");
		let EPMShowRestart = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ShowRestart");
		let EPMrestartLabel = document.getElementById("EPMstring-bundle").getString("EPMRestart");
		let EPMopenApp = window['BrowserOpenAddonsMgr'] || window['openAddonsMgr'] || window['toEM']; 
		let EPMscrollInToView = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.ScrollInToView");
		let EPMhideSigned = Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.HideSigned");
		let EPMBlackList = Services.prefs.getCharPref("extensions.ExtensionPreferencesMenu.BlackList");
		if (EPMBlackList) {
			EPMBlackList = EPMBlackList.split("").reverse().join("");
			EPMBlackListArray = EPMBlackList.split(/\,(?!\\)/);
			for (let i = 0; i < EPMBlackListArray.length; i++)
				EPMBlackListArray[i] = EPMBlackListArray[i].split("").reverse().join("").replace(/^ | $/g,"").replace(/\\\,/g,",").toLowerCase();
		}

		AddonManager.getAllAddons(function(addon) {
			while (aParent.childNodes.length)
				aParent.removeChild(aParent.firstChild);

			addon.forEach(function(EPMaddon) {
				if (EPMaddon.type == "extension" && (!EPMaddon.appDisabled || (EPMaddon.appDisabled && EPMshowIncompatible)) && ((EPMaddon.isActive && EPMaddon.optionsURL) || ((EPMaddon.userDisabled && EPMshowDisabled) || (!EPMaddon.userDisabled && EPMshowAll) || (EPMaddon.appDisabled && EPMshowIncompatible)))) {
					item = document.createElement("menuitem");
					item.setAttribute("label", EPshowVersion ? EPMaddon.name + "  " + (EPMhideSigned ? EPMaddon.version.replace(".1-signed", ".1") : EPMaddon.version) : EPMaddon.name);
					if (EPMaddon.description)
						item.setAttribute("tooltiptext", EPMaddon.description);
					item.addEventListener("command", function(event) {
						if (EPMaddon.optionsURL && !event.ctrlKey) {
							switch (EPMaddon.optionsType) {
								case "2":
									EPMopenApp('addons://detail/' + encodeURIComponent(EPMaddon.id) + (EPMscrollInToView ? '/preferences' : ''));
									break;
								case "3":
									"switchToTabHavingURI" in window ? switchToTabHavingURI(EPMaddon.optionsURL, true) : openTab("contentTab", { contentPage: EPMaddon.optionsURL });
									break;
								default:
									openDialog(EPMaddon.optionsURL, EPMaddon.name, 'chrome,titlebar,toolbar,resizable,scrollbars,centerscreen,dialog=no,modal=no');
								}
							let EPMpanelPopup = document.getElementById('PanelUI-popup');
							if (EPMpanelPopup)
								if (EPMpanelPopup.state == 'open')
									EPMpanelPopup.hidePopup(); 
						}
					});
					item.addEventListener("click", function(event) {
						if (EPMquickActions) {
							if (event.button == 1 && EPMaddon.homepageURL)
								"gBrowser" in window ? gBrowser.addTab(EPMaddon.homepageURL) : Cc["@mozilla.org/messenger;1"].createInstance(Ci.nsIMessenger).launchExternalURL(EPMaddon.homepageURL);
							if (event.button == 2 || (event.button == 0 && event.ctrlKey))
								AddonManager.getAddonByID(EPMaddon.id, function(addon) {
									addon.userDisabled = !addon.userDisabled;
									if (addon.operationsRequiringRestart && !EPMShowRestart)
										Application.restart();
									ExtensionPreferencesMenu.setup();
								});
						}
					});
					if (!EPMaddon.optionsURL)
						item.setAttribute("disabled", true);
					if (!EPMaddon.isActive)
						item.setAttribute("EPMisDisabled", true);
					if (EPMaddon.pendingOperations == 1)
						item.setAttribute("EPMtoBeToggled", true);
					if (EPMaddon.pendingOperations == 2)
						item.setAttribute("EPMtoBeToggled", false);

					if (EPshowIcon) {
						item.setAttribute("class", "menuitem-iconic");
						MenuIconURL = EPMaddon.iconURL || 'chrome://mozapps/skin/extensions/extensionGeneric-16.png';
						item.setAttribute("style", "list-style-image: url(" + MenuIconURL + ")");
					}
					else
						item.setAttribute("class", "menuitem");

					if (EPMBlackListArray)
						for (let i = 0; i < EPMBlackListArray.length; i++)
							if (EPMBlackListArray[i] == EPMaddon.name.toLowerCase()) {
								EPMShowIt = false;
								break;
							}
							else
								EPMShowIt = true;

					if (EPMShowIt)
						aParent.appendChild(item);
				}
			});

			Cc["@mozilla.org/xul/xul-sort-service;1"].getService(Ci.nsIXULSortService).sort(aParent, "label", "ascending");

			if (EPMShowRestart) {
				aParent.appendChild(document.createElement("menuseparator"));

				item = document.createElement("menuitem");
				item.setAttribute("label", EPMrestartLabel);
				item.addEventListener("command", function() { Application.restart() });

				if (EPshowIcon) {
					item.setAttribute("class", "menuitem-iconic");
					item.setAttribute("style", "list-style-image: url(chrome://prefsmenu-default/skin/restart.png)");
				}
				else
					item.setAttribute("class", "menuitem");

				aParent.appendChild(item);
			}
		});
	},

	checkMiddleClick: function(aEvent) {
		aEvent.preventDefault();
		(window['BrowserOpenAddonsMgr'] || window['openAddonsMgr'] || window['toEM'])((Services.prefs.getBoolPref("extensions.ExtensionPreferencesMenu.OpenExtensionsPage")) ? 'addons://list/extension' : '');
	},

	shutdown: function() {
		window.removeEventListener('unload', function EPMunload() { ExtensionPreferencesMenu.shutdown() });
		AddonManager.removeAddonListener(this);
		this._branch.removeObserver("", this);
		"CustomizableUI" in window ? CustomizableUI.removeListener(this) : window.removeEventListener('customizationchange', function EPMsetup() { ExtensionPreferencesMenu.setup() });
	}
};

window.addEventListener('load', function EPMload() { ExtensionPreferencesMenu.startup() });
window.addEventListener('unload', function EPMunload() { ExtensionPreferencesMenu.shutdown() });
